/*
$Id: TvtShaper/tvtshaper40.inc 1.11 2007/11/05 11:46:01EST Popov, Chavdar (Chavdar Popov) Dev  $
$ProjectName: d:/elect/source/device_descriptions/dd_library/common/lr_food3_hart_device_descriptions_common_library.pj $   $Locker:  $
*/
#ifndef TVTSHAPER40_INC
#define TVTSHAPER40_INC
#endif
/*                                                       For Internal Use Only.
   Description ----------------------------------------------------------------
       Library File For 40 point TVT Shaper.

       This is an EDD feature to support SMPI Standard 40 point manual TVT shapers

       The only method to use is "method_menu_tvt_shaper" the remaining items
       are support elements

         The basic structure:
                             method_menu_tvt_shaper
                                        |
                             offlineDialog_tvt_shaper
                                        |
                                -----------------
                                |               |
                Page_tvt_shaper_graph        Page_tvt_shaper_list
                        |
             ---------------------------------------------------
             |                        |                        |
      graph_tvt_shaper      Group_tvt_presettings     Group_tvt_shaper,

       The "trendopt.dd" file must be created.
       Use the template with the name "trendopt.tpl.dd".

         Copyright (C) Siemens-Milltronics Process Instruments Inc.
         All Rights Reserved.

   Warnings -------------------------------------------------------------------

   Special Revision Notes -----------------------------------------------------
      To add this feature to a DD:

      1) Add "method_menu_tvt_shaper" under the "Device" menu
      2) Make sure the defines the options file reference the correct variables
      3) Make sure the dictionary entries from the options file are defined
      4) Include "tvtshaper40.inc" (this file) in the main ddl file
      5) Add "tvtshaper40.inc" (this file) to the devices file
      6) Add "tvtshaper40opt.dd" to the devices file

   Defines --------------------------------------------------------------------*/
#define NUMBER_TVT_SHAPER_POINTS          40

// Tvt shaper modes
#define TVT_SHAPER_MODE_OFF 0
#define TVT_SHAPER_MODE_ON  1

/* Include Files --------------------------------------------------------------*/
#include "tvtShaper40Opt.dd"

/* Collections ----------------------------------------------------------------*/

/* Relations (Unit, Array, Refresh) -------------------------------------------*/
REFRESH refresh_tvt_shaper_array
{
   var_local_tvt_shaper_index_display
   :
   var_local_tvt_shaper_value_display
}

/* Variable Definitions -------------------------------------------------------*/
var_backup_tvt_shaper_mode     LIKE VARIABLE TVT_SHAPER_MODE  {}

ARRAY array_backup_tvt_shaper_points
{
   TYPE                TVT_SHAPER_Y_MIN;
   NUMBER_OF_ELEMENTS  NUMBER_TVT_SHAPER_POINTS;
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_profile_taken
{
   CLASS       LOCAL;
   TYPE        UNSIGNED_INTEGER(1)
   {
      DEFAULT_VALUE FALSE;
   }
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_index_display
{
   LABEL       VAR_LOCAL_TVT_SHAPER_INDEX_LABEL;
   HELP        VAR_LOCAL_TVT_SHAPER_INDEX_HELP;
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      DEFAULT_VALUE 1;
      MIN_VALUE     1;
      MAX_VALUE     NUMBER_TVT_SHAPER_POINTS;
   }
   POST_EDIT_ACTIONS
   {
       method_get_tvt_shaper_index_display
   }
   HANDLING    READ & WRITE;
}

/*-----------------------------------------------------------------------------
   Description: The value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_value_display
{
   LABEL       VAR_LOCAL_TVT_SHAPER_VALUE_LABEL;
   HELP        VAR_LOCAL_TVT_SHAPER_VALUE_HELP;
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      DEFAULT_VALUE 0;
      MIN_VALUE     TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE;
      MAX_VALUE     TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE;
   }
   PRE_EDIT_ACTIONS
   {
       method_set_tvt_shaper_offset_display
   }
   POST_EDIT_ACTIONS
   {
       method_get_tvt_shaper_offset_display
   }
   HANDLING    READ & WRITE;
   CONSTANT_UNIT "dB";
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_zero_offset
{
   CLASS LOCAL;
   TYPE  FLOAT;
}

VARIABLE var_local_tvt_shaper_step_size
{
   CLASS LOCAL;
   TYPE  FLOAT;
}

VARIABLE var_local_marker_start
{
   CLASS LOCAL;
   TYPE  INTEGER(2)
   {
      DEFAULT_VALUE 0;
   }
}

VARIABLE var_local_marker_end
{
   CLASS LOCAL;
   TYPE  INTEGER(2)
   {
      DEFAULT_VALUE 1;
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
ARRAY array_local_tvt_point
{
    TYPE local_data_point_y;
    NUMBER_OF_ELEMENTS NUMBER_TVT_POINTS;
}

/* Command Definitions --------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/

/* Menu Definitions -----------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: Displays the tvt shaper dialog
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU offlineDialog_tvt_shaper
{
   LABEL MENU_DIALOG_TVT_SHAPER_LABEL;
   STYLE_OFFLINE_DIALOG
   ITEMS
   {
      Page_tvt_shaper_graph,
      Page_tvt_shaper_list_01_20,
      Page_tvt_shaper_list_21_40
   }
}

/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a graphical editing interface
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Page_tvt_shaper_graph
{
   LABEL      MENU_PAGE_TVT_SHAPER_GRAPH_LABEL;
   STYLE_PAGE
   ITEMS
   {
      #ifndef HELP_SUPPORT
         graph_tvt_shaper,
      #endif
      Group_tvt_presettings,
      Group_tvt_shaper
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for refreshing the profiles
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Group_tvt_presettings
{
   LABEL      MENU_GROUP_TVT_PRESETTINGS_LABEL;
   STYLE_GROUP
   ITEMS
   {
      method_tvt_shaper_update_from_device
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for editing the TVT
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Group_tvt_shaper
{
   LABEL      MENU_GROUP_TVT_SHAPER_LABEL;
   STYLE_GROUP
   ITEMS
   {
      var_local_tvt_shaper_index_display,
      method_tvt_index_plus,
      method_tvt_index_minus,
      COLUMNBREAK,
      var_local_tvt_shaper_value_display,
      method_tvt_value_increase,
      method_tvt_value_decrease
   }
}

/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a list of tvt points to be edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Page_tvt_shaper_list_01_20
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "1 - 20";
   STYLE_PAGE
   ITEMS
   {
      TVT_SHAPER_ARRAY[ 0] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 1] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 2] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 3] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 4] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 5] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 6] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 7] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 8] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 9] (READ_ONLY),
      COLUMNBREAK,
      TVT_SHAPER_ARRAY[10] (READ_ONLY),
      TVT_SHAPER_ARRAY[11] (READ_ONLY),
      TVT_SHAPER_ARRAY[12] (READ_ONLY),
      TVT_SHAPER_ARRAY[13] (READ_ONLY),
      TVT_SHAPER_ARRAY[14] (READ_ONLY),
      TVT_SHAPER_ARRAY[15] (READ_ONLY),
      TVT_SHAPER_ARRAY[16] (READ_ONLY),
      TVT_SHAPER_ARRAY[17] (READ_ONLY),
      TVT_SHAPER_ARRAY[18] (READ_ONLY),
      TVT_SHAPER_ARRAY[19] (READ_ONLY)
   }
}

MENU Page_tvt_shaper_list_21_40
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "21 - 40";
   STYLE_PAGE
   ITEMS
   {
      TVT_SHAPER_ARRAY[20] (READ_ONLY),
      TVT_SHAPER_ARRAY[21] (READ_ONLY),
      TVT_SHAPER_ARRAY[22] (READ_ONLY),
      TVT_SHAPER_ARRAY[23] (READ_ONLY),
      TVT_SHAPER_ARRAY[24] (READ_ONLY),
      TVT_SHAPER_ARRAY[25] (READ_ONLY),
      TVT_SHAPER_ARRAY[26] (READ_ONLY),
      TVT_SHAPER_ARRAY[27] (READ_ONLY),
      TVT_SHAPER_ARRAY[28] (READ_ONLY),
      TVT_SHAPER_ARRAY[29] (READ_ONLY),
      COLUMNBREAK,
      TVT_SHAPER_ARRAY[30] (READ_ONLY),
      TVT_SHAPER_ARRAY[31] (READ_ONLY),
      TVT_SHAPER_ARRAY[32] (READ_ONLY),
      TVT_SHAPER_ARRAY[33] (READ_ONLY),
      TVT_SHAPER_ARRAY[34] (READ_ONLY),
      TVT_SHAPER_ARRAY[35] (READ_ONLY),
      TVT_SHAPER_ARRAY[36] (READ_ONLY),
      TVT_SHAPER_ARRAY[37] (READ_ONLY),
      TVT_SHAPER_ARRAY[38] (READ_ONLY),
      TVT_SHAPER_ARRAY[39] (READ_ONLY)
   }
}

/* Visual component Definitions -----------------------------------------------*/

/*-----------------------------------------------------------------------------
   Description: Displays graph area, content is referenced from here
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
#ifndef HELP_SUPPORT
   GRAPH graph_tvt_shaper
   {
       HEIGHT     X_LARGE;
       WIDTH      LARGE;
       X_AXIS     x_axis_tvt_shaper;
       MEMBERS
       {
           WAVEFORM_TVT_SHAPER_MARKER_START, waveform_tvt_shaper_marker_start;
           WAVEFORM_TVT_SHAPER_MARKER_END,   waveform_tvt_shaper_marker_end;
           WAVEFORM_TVT_SHAPER_TVT,          waveform_tvt_shaper_tvt_profile;
           WAVEFORM_TVT_SHAPER_ECHO,         waveform_tvt_shaper_echo_profile;
       }
       VALIDITY ( var_local_tvt_shaper_profile_taken == TRUE );
   }

   WAVEFORM waveform_tvt_shaper_echo_profile
   {
      Y_AXIS     y_axis_tvt_shaper;
      LINE_COLOR TVT_SHAPER_ECHO_PROFILE_COLOR;
      TYPE YT
      {
         X_INITIAL        var_local_tvt_shaper_x_axis_echo_start;
         X_INCREMENT      var_local_tvt_shaper_x_axis_echo_increment;
         NUMBER_OF_POINTS TVT_SHAPER_ECHO_PROFILE_SIZE;
         Y_VALUES
         {
            TVT_SHAPER_ECHO_PROFILE_ARRAY
         }
      }
   }

   WAVEFORM waveform_tvt_shaper_tvt_profile
   {
      Y_AXIS     y_axis_tvt_shaper;
      LINE_COLOR TVT_SHAPER_TVT_PROFILE_COLOR;
      TYPE YT
      {
         X_INITIAL        var_local_tvt_shaper_tvt_start;
         X_INCREMENT      var_local_tvt_shaper_tvt_increment;
         NUMBER_OF_POINTS TVT_SHAPER_TVT_PROFILE_SIZE;
         Y_VALUES
         {
            array_local_tvt_point
         }
      }
   }

   WAVEFORM waveform_tvt_shaper_marker_start
   {
      Y_AXIS     y_axis_tvt_shaper;
      LINE_COLOR 0x000000;
      TYPE XY
      {
         X_VALUES
         {
            var_local_marker_start, var_local_marker_start
         }
         Y_VALUES
         {
            TVT_SHAPER_Y_MAX, TVT_SHAPER_Y_MIN 
         }
         NUMBER_OF_POINTS 2;
      }
   }

   WAVEFORM waveform_tvt_shaper_marker_end
   {
      Y_AXIS     y_axis_tvt_shaper;
      LINE_COLOR 0x000000;
      TYPE XY
      {
         X_VALUES
         {
            var_local_marker_end, var_local_marker_end
         }
         Y_VALUES
         {
            TVT_SHAPER_Y_MAX, TVT_SHAPER_Y_MIN 
         }
         NUMBER_OF_POINTS 2;
      }
   }

   AXIS x_axis_tvt_shaper
   {
      LABEL         X_AXIS_TVT_SHAPER_LABEL;
      MIN_VALUE     var_local_tvt_shaper_x_axis_echo_start;
      MAX_VALUE     NUMBER_TVT_SHAPER_POINTS + 1;
      CONSTANT_UNIT X_AXIS_TVT_SHAPER_UNITS;
   }

   AXIS y_axis_tvt_shaper
   {
       LABEL Y_AXIS_TVT_SHAPER_LABEL;
       MIN_VALUE TVT_SHAPER_Y_MIN;
       MAX_VALUE TVT_SHAPER_Y_MAX;
       CONSTANT_UNIT Y_AXIS_TVT_SHAPER_UNITS;
   }
#endif

/* Method Definitions ---------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: This method is displayed under the "Device" menu
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_menu_tvt_shaper
{
   LABEL      METHOD_MENU_TVT_SHAPER_LABEL;
   HELP       METHOD_MENU_TVT_SHAPER_HELP;
   ACCESS     OFFLINE;
   CLASS      INPUT;
   DEFINITION
   {
      long dialogResult;
      int  point;

      std_CheckDeviceIdentification();
      if (ret_CheckDeviceIdentification == ID_CHECK_SUCCESS)
         {
         // Make a local copy so we can restore data
         method_backup_tvt_shaper();

         // Add an abort method so we can restore the data
         add_abort_method(method_restore_tvt_shaper);

         // Grab an initial echo profile to use as a reference
         method_tvt_shaper_measure();

         // Turn on the mode because points may base validity on mode
         TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;

         // Display the dialog
         MenuDisplay(offlineDialog_tvt_shaper, TVT_SHAPER_DIALOG_BUTTONS, dialogResult);

         // Dialog was closed, continue based on user selection
         switch( dialogResult )
            {
            case 0:  // APPLY_BUTTON
               method_tvt_shaper_transfer_to_device();
               break;
            case 1:  // CANCEL_BUTTON
               method_restore_tvt_shaper();
            }

         // Remove the abort method that was added earlier
         remove_abort_method(method_restore_tvt_shaper);
         }
   }
}

/*-----------------------------------------------------------------------------
   Description: Make a backup copy of the data we are modifying so it can be
                restored if needed
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_backup_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Make a local copy so we can restore data
      var_backup_tvt_shaper_mode = TVT_SHAPER_MODE;

      for (point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++)
      {
         array_backup_tvt_shaper_points[point] = TVT_SHAPER_ARRAY[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description: Restore the data we are modifying from the backup copy
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_restore_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Restore data from local copy
      TVT_SHAPER_MODE = var_backup_tvt_shaper_mode;

      for (point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++)
      {
         TVT_SHAPER_ARRAY[point] = array_backup_tvt_shaper_points[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_index_display
{
   DEFINITION
   {
      if (var_local_tvt_shaper_index_display > var_local_tvt_shaper_index_display.MAX_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MAX_VALUE;
      }
      if (var_local_tvt_shaper_index_display < var_local_tvt_shaper_index_display.MIN_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MIN_VALUE;
      }
      var_local_marker_start = var_local_tvt_shaper_index_display - 1;
      var_local_marker_end   = var_local_tvt_shaper_index_display;
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_set_tvt_shaper_offset_display
{
   DEFINITION
   {
      var_local_tvt_shaper_value_display = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index_display - 1] * TVT_SHAPER_STEP_SIZE;
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_offset_display
{
   DEFINITION
   {
      int i;
      int index_start;
      int index_end;

      // make sure value is valid
      if (var_local_tvt_shaper_value_display > (TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE))
      {
         var_local_tvt_shaper_value_display = (TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE);
      }
      if (var_local_tvt_shaper_value_display < (TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE))
      {
         var_local_tvt_shaper_value_display = (TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE);
      }
      TVT_SHAPER_ARRAY[var_local_tvt_shaper_index_display - 1] = var_local_tvt_shaper_value_display / TVT_SHAPER_STEP_SIZE;

      // Transfer new value into TVT
      index_start = ((var_local_tvt_shaper_index_display - 1) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset;
      index_end   = ((var_local_tvt_shaper_index_display    ) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset;

      for (i = index_start; i < index_end; i++)
         {
         if ((i >= 0) && (i < TVT_SHAPER_TVT_PROFILE_SIZE))
            {
            array_local_tvt_point[i] = TVT_SHAPER_TVT_PROFILE_ARRAY[i] + var_local_tvt_shaper_value_display;
            }
         }
      
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_plus
{
   LABEL      METHOD_TVT_INDEX_PLUS_LABEL;
   HELP       METHOD_TVT_INDEX_PLUS_HELP;
   DEFINITION
   {
      var_local_tvt_shaper_index_display++;
      method_get_tvt_shaper_index_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_minus
{
   LABEL      METHOD_TVT_INDEX_MINUS_LABEL;
   HELP       METHOD_TVT_INDEX_MINUS_HELP;
   DEFINITION
   {
      var_local_tvt_shaper_index_display--;
      method_get_tvt_shaper_index_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_increase
{
   LABEL      METHOD_TVT_VALUE_INCREASE_LABEL;
   HELP       METHOD_TVT_VALUE_INCREASE_HELP;
   DEFINITION
   {
      var_local_tvt_shaper_value_display++;
      method_get_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_decrease
{
   LABEL       METHOD_TVT_VALUE_DECREASE_LABEL;
   HELP        METHOD_TVT_VALUE_DECREASE_HELP;
   DEFINITION
   {
      var_local_tvt_shaper_value_display--;
      method_get_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_measure
{
  DEFINITION
  {
      int tvt_shaper_mode;
      float profile_increment;
      int next_increment_index;
      int current_index;
      int tvt_point_index;

      var_local_tvt_shaper_profile_taken = FALSE;

      CMD_READ_TVT_SHAPER_MODE;
      tvt_shaper_mode = TVT_SHAPER_MODE;
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_OFF;
      CMD_WRITE_TVT_SHAPER_MODE;

      // Delay increased from 2 seconds to 10 seconds to wait for a valid profile when rate is slow on PA!
      DELAY_TIME(10);

      METHOD_TVT_SHAPER_READ_PROFILE_DATA();

      TVT_SHAPER_MODE = tvt_shaper_mode;
      CMD_WRITE_TVT_SHAPER_MODE;

      // Turn on the mode because points may base validity on mode
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;

      // Compute echo profile values
      profile_increment                          = (TVT_SHAPER_X_MAX - TVT_SHAPER_X_MIN) / TVT_SHAPER_ECHO_PROFILE_SIZE;
      var_local_tvt_shaper_x_axis_echo_increment = NUMBER_TVT_SHAPER_POINTS / (TVT_SHAPER_X_MAX / profile_increment);
      var_local_tvt_shaper_x_axis_echo_start     = (TVT_SHAPER_X_MIN / profile_increment) * var_local_tvt_shaper_x_axis_echo_increment;

      // Compute tvt values
      profile_increment                  = (TVT_SHAPER_X_MAX - TVT_SHAPER_X_MIN) / TVT_SHAPER_TVT_PROFILE_SIZE;
      var_local_tvt_shaper_tvt_increment = NUMBER_TVT_SHAPER_POINTS / (TVT_SHAPER_X_MAX / profile_increment);
      var_local_tvt_shaper_tvt_start     = (TVT_SHAPER_X_MIN / profile_increment) * var_local_tvt_shaper_tvt_increment;

      var_local_tvt_shaper_zero_offset = - var_local_tvt_shaper_tvt_start / var_local_tvt_shaper_tvt_increment;
      var_local_tvt_shaper_step_size = (TVT_SHAPER_TVT_PROFILE_SIZE - var_local_tvt_shaper_zero_offset) / NUMBER_TVT_SHAPER_POINTS;

      // update local copy of tvt
      tvt_point_index = -1;
      next_increment_index = var_local_tvt_shaper_zero_offset;
      for (current_index = 0; current_index < TVT_SHAPER_TVT_PROFILE_SIZE; current_index++)
         {
         if (current_index >= next_increment_index)
            {
            tvt_point_index++;
            next_increment_index = ((tvt_point_index + 1) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset;
            }

         if ((tvt_point_index < 0) || (tvt_point_index >= NUMBER_TVT_SHAPER_POINTS))
            {
            array_local_tvt_point[current_index] = TVT_SHAPER_TVT_PROFILE_ARRAY[current_index];
            }
         else
            {
            array_local_tvt_point[current_index] = TVT_SHAPER_TVT_PROFILE_ARRAY[current_index] + TVT_SHAPER_ARRAY[tvt_point_index] * TVT_SHAPER_STEP_SIZE;
            }
         }

      var_local_tvt_shaper_profile_taken = TRUE;
  }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper to the device
                It writes both the mode and the points
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_transfer_to_device
{
   LABEL METHOD_TRANSFER_TVT_SHAPER_TO_DEVICE_LABEL;
   HELP  METHOD_TRANSFER_TVT_SHAPER_TO_DEVICE_HELP;
   DEFINITION
   {
      int point;

      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;
      CMD_WRITE_TVT_SHAPER_MODE;
      CMD_WRITE_TVT_SHAPER_POINTS;
      PUT_MESSAGE(TVT_SHAPER_SUCCESSFUL_LABEL);
   }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper from the device
                It reads both the mode and the points
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_update_from_device
{
   LABEL METHOD_TVT_SHAPER_UPDATE_FROM_DEVICE_LABEL;
   HELP  METHOD_TVT_SHAPER_UPDATE_FROM_DEVICE_HELP;   
   
   DEFINITION
   {
      CMD_READ_TVT_SHAPER_MODE;
      CMD_READ_TVT_SHAPER_POINTS;
      method_tvt_shaper_measure();
   }
}

/*-----------------------------------------------------------------------------
                                                         For Internal Use Only.
      Copyright (C) Siemens-Milltronics Process Instruments Inc.
      All Rights Reserved.

End of file -------------------------------------------------------------------*/
